/*
 * IBM SupportPac MA0K
 * WebSphere MQ Completion Code and Reason Code analysis programs V1.4
 * http://www.ibm.com/support/docview.wss?uid=swg24000652
 *
 * MQCM - 'WebSphere MQ' Code to Message conversion functions
 *
 * "mqcm.c" function module in C
 *
 * Source file: \\IBM-L3V1408\WMQ-PROJ\MA0K-v1.4-Build\_Source\mqcm-14.c
 *
 * Environments: Windows 32 bit, Unix (AIX, HP-UX, Solaris, Linux), z/OS
 *
 * Revision History
 * ================
 * Version  Date         Author    Changes
 *     1.1  20 Apr 1999  Glenn     Initial version for MQSeries 5.0
 *                       Baddeley <glenbadd@au1.ibm.com>
 *       2  14 Feb 2000  GB        Updated for MQSeries 5.1 using
 *                                 cmqc.h supplied for HP-UX,
 *                                 Make literals more compact
 *       3  28 Feb 2000  GB        Handle Reason Codes MQRC_using,
 *                                 Dont reference MQRC_APPL if MQVER50 defined
 *       4  01 May 2000  GB        Check MQRC_APPL defined before referencing
 *       5  22 Nov 2000  GB        Updated for MQSeries 5.2 using
 *                                 cmqc.h supplied for Windows NT,
 *                                 Change 2064-2081 to 2164-2181_UNDEFINED,
 *                                 Reduce storage usage for undefined codes,
 *                                 Make reentrant by accepting pointer to return
 *                                 strings, Document functions
 *       6  15 Feb 2001  GB        Add 6000-6129 Reason Codes for MQSeries 5.2 C++
 *       7  09 Oct 2001  GB        Change MQRC_PAGESET_FULL (for OS/390)
 *                                 to MQRC_STORAGE_MEDIUM_FULL (all platforms)
 *       8  11 Jul 2002  GB        Add Reason Codes 2362-2402 for WebSphere MQ 5.3,
 *                                 Use 'MIN' defines rather than 'OFFSET' defines
 *       9  25 Nov 2003  GB        Fix bug: Add comma after literal for 2361
 *      10  28 Jan 2004  GB        Change copyright owner from IBM GSA to IBM Australia
 *      11  10 Mar 2004  GB        Add USS (Unix System Services) supported
 *                                 Add more comments on history of Reason Codes
 *                                 Change 2120 TRUNCATED to CONVERTED_MSG_TOO_BIG to
 *                                 match name in WMQ Messages manual
 *      12  01 Oct 2004  GB        Add stdio.h for islower() and toupper()
 *      13  25 Oct 2005  GB        Updated for WMQ 6.0 to add 2210 - 2212,
 *                                 2228 - 2229, 2395, 2406 - 2423
 *      14  25 Sep 2008  GB        Updated for WMQ 7.0 GA to add 2200, 2372,
 *                                 2408, 2424 - 2542
 *                                 Revise copyright and owner information
 */
#define SRCID "@(#)MQCM.C VER=1.14 DATE=25-Sep-2008"

/*  <N_OCO_COPYRIGHT>
 *  Licensed Materials - Property of IBM
 *  (c) Copyright IBM Corp. 2008 All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *  <NOC_COPYRIGHT>                                                   */
#define OWNID "@(#)Licensed Materials - Property of IBM"
#define CPRID "@(#)Copyright 2008 Corp. All Rights Reserved"

/* Includes */
#include <string.h>
#include <stdio.h>  /* For islower(), toupper() */
 
/* Define MQ data types and symbols */
#include <cmqc.h>

/* Define MQCM_ symbols */
#include "mqcmh.h"

/* Local data */
static char SrcId[] = SRCID, CprId[] = CPRID, OwnId[] = OWNID;

static char *CcLits[] = 
 {
  "UNKNOWN",  /* -1 */
  "OK",
  "WARNING",
  "FAILED",   /* 2 */
 };

static char *RcLits2[] = 
 {
"ALIAS_BASE_Q_TYPE_ERROR",   /* 2001 */
"ALREADY_CONNECTED",
"BACKED_OUT",
"BUFFER_ERROR",
"BUFFER_LENGTH_ERROR",
"CHAR_ATTR_LENGTH_ERROR",
"CHAR_ATTRS_ERROR",
"CHAR_ATTRS_TOO_SHORT",
"CONNECTION_BROKEN",
"DATA_LENGTH_ERROR",
"DYNAMIC_Q_NAME_ERROR",
"ENVIRONMENT_ERROR",
"EXPIRY_ERROR",
"FEEDBACK_ERROR",
NULL,
"GET_INHIBITED",
"HANDLE_NOT_AVAILABLE",
"HCONN_ERROR",
"HOBJ_ERROR",
"INHIBIT_VALUE_ERROR",
"INT_ATTR_COUNT_ERROR",
"INT_ATTR_COUNT_TOO_SMALL",
"INT_ATTRS_ARRAY_ERROR",
"SYNCPOINT_LIMIT_REACHED",
"MAX_CONNS_LIMIT_REACHED",
"MD_ERROR",
"MISSING_REPLY_TO_Q",
NULL,
"MSG_TYPE_ERROR",
"MSG_TOO_BIG_FOR_Q",
"MSG_TOO_BIG_FOR_Q_MGR",
NULL,
"NO_MSG_AVAILABLE",
"NO_MSG_UNDER_CURSOR",
"NOT_AUTHORIZED",
"NOT_OPEN_FOR_BROWSE",
"NOT_OPEN_FOR_INPUT",
"NOT_OPEN_FOR_INQUIRE",
"NOT_OPEN_FOR_OUTPUT",
"NOT_OPEN_FOR_SET",
"OBJECT_CHANGED",
"OBJECT_IN_USE",
"OBJECT_TYPE_ERROR",
"OD_ERROR",
"OPTION_NOT_VALID_FOR_TYPE",
"OPTIONS_ERROR",
"PERSISTENCE_ERROR",
"PERSISTENT_NOT_ALLOWED",
"PRIORITY_EXCEEDS_MAXIMUM",
"PRIORITY_ERROR",
"PUT_INHIBITED",
"Q_DELETED",
"Q_FULL",
NULL,
"Q_NOT_EMPTY",
"Q_SPACE_NOT_AVAILABLE",
"Q_TYPE_ERROR",
"Q_MGR_NAME_ERROR",
"Q_MGR_NOT_AVAILABLE",
NULL,
"REPORT_OPTIONS_ERROR",
"SECOND_MARK_NOT_ALLOWED",
"SECURITY_ERROR",
NULL,
"SELECTOR_COUNT_ERROR",
"SELECTOR_LIMIT_EXCEEDED",
"SELECTOR_ERROR",
"SELECTOR_NOT_FOR_TYPE",
"SIGNAL_OUTSTANDING",
"SIGNAL_REQUEST_ACCEPTED",
"STORAGE_NOT_AVAILABLE",
"SYNCPOINT_NOT_AVAILABLE",
NULL,
NULL,
"TRIGGER_CONTROL_ERROR",
"TRIGGER_DEPTH_ERROR",
"TRIGGER_MSG_PRIORITY_ERR",
"TRIGGER_TYPE_ERROR",
"TRUNCATED_MSG_ACCEPTED",
"TRUNCATED_MSG_FAILED",
NULL,
"UNKNOWN_ALIAS_BASE_Q",
NULL,
NULL,
"UNKNOWN_OBJECT_NAME",
"UNKNOWN_OBJECT_Q_MGR",
"UNKNOWN_REMOTE_Q_MGR",
NULL,
NULL,
"WAIT_INTERVAL_ERROR",
"XMIT_Q_TYPE_ERROR",
"XMIT_Q_USAGE_ERROR",
"NOT_OPEN_FOR_PASS_ALL",
"NOT_OPEN_FOR_PASS_IDENT",
"NOT_OPEN_FOR_SET_ALL",
"NOT_OPEN_FOR_SET_IDENT",
"CONTEXT_HANDLE_ERROR",
"CONTEXT_NOT_AVAILABLE",
"SIGNAL1_ERROR",
"OBJECT_ALREADY_EXISTS",     /* 2100 */
"OBJECT_DAMAGED",
"RESOURCE_PROBLEM",
"ANOTHER_Q_MGR_CONNECTED",
"UNKNOWN_REPORT_OPTION",
"STORAGE_CLASS_ERROR",
"COD_NOT_VALID_FOR_XCF_Q",
"XWAIT_CANCELED",
"XWAIT_ERROR",
"SUPPRESSED_BY_EXIT",
"FORMAT_ERROR",
"SOURCE_CCSID_ERROR",
"SOURCE_INTEGER_ENC_ERROR",
"SOURCE_DECIMAL_ENC_ERROR",
"SOURCE_FLOAT_ENC_ERROR",
"TARGET_CCSID_ERROR",
"TARGET_INTEGER_ENC_ERROR",
"TARGET_DECIMAL_ENC_ERROR",
"TARGET_FLOAT_ENC_ERROR",
"NOT_CONVERTED",
"CONVERTED_MSG_TOO_BIG",     /* 2120, also TRUNCATED */
"NO_EXTERNAL_PARTICIPANTS",
"PARTICIPANT_NOT_AVAILABLE",
"OUTCOME_MIXED",
"OUTCOME_PENDING",
"BRIDGE_STARTED",
"BRIDGE_STOPPED",
"ADAPTER_STORAGE_SHORTAGE",
"UOW_IN_PROGRESS",
"ADAPTER_CONN_LOAD_ERROR",
"ADAPTER_SERV_LOAD_ERROR",
"ADAPTER_DEFS_ERROR",
"ADAPTER_DEFS_LOAD_ERROR",
"ADAPTER_CONV_LOAD_ERROR",
"BO_ERROR",
"DH_ERROR",
"MULTIPLE_REASONS",
"OPEN_FAILED",
"ADAPTER_DISC_LOAD_ERROR",
"CNO_ERROR",
"CICS_WAIT_FAILED",
"DLH_ERROR",
"HEADER_ERROR",
"SOURCE_LENGTH_ERROR",
"TARGET_LENGTH_ERROR",
"SOURCE_BUFFER_ERROR",
"TARGET_BUFFER_ERROR",
NULL,
"IIH_ERROR",
"PCF_ERROR",
"DBCS_ERROR",
NULL,
"OBJECT_NAME_ERROR",
"OBJECT_Q_MGR_NAME_ERROR",
"RECS_PRESENT_ERROR",
"OBJECT_RECORDS_ERROR",
"RESPONSE_RECORDS_ERROR",
"ASID_MISMATCH",
"PMO_RECORD_FLAGS_ERROR",
"PUT_MSG_RECORDS_ERROR",
"CONN_ID_IN_USE",
"Q_MGR_QUIESCING",
"Q_MGR_STOPPING",
"DUPLICATE_RECOV_COORD",
NULL,   /* 2164 */
NULL,
NULL,
NULL,
NULL,
NULL,
NULL,
NULL,
NULL,
"PMO_ERROR",
NULL,   /* 2174 */
NULL,
NULL,
NULL,
NULL,
NULL,
NULL,
NULL,
"API_EXIT_NOT_FOUND",
"API_EXIT_LOAD_ERROR",
"REMOTE_Q_NAME_ERROR",
"INCONSISTENT_PERSISTENCE",
"GMO_ERROR",
"CICS_BRIDGE_RESTRICTION",   /* 2187 - 2190, new for MQ 5.1 */
"STOPPED_BY_CLUSTER_EXIT",
"CLUSTER_RESOLUTION_ERROR",
"CONVERTED_STRING_TOO_BIG",
"TMC_ERROR",
"STORAGE_MEDIUM_FULL",       /* 2192, also PAGESET_FULL on S/390 */
"PAGESET_ERROR",
"NAME_NOT_VALID_FOR_TYPE",
"UNEXPECTED_ERROR",
"UNKNOWN_XMIT_Q",
"UNKNOWN_DEF_XMIT_Q",
"DEF_XMIT_Q_TYPE_ERROR",
"DEF_XMIT_Q_USAGE_ERROR",
"MSG_MARKED_BROWSE_CO_OP",   /* 2200, new for MQ 7.0 */
"NAME_IN_USE",
"CONNECTION_QUIESCING",
"CONNECTION_STOPPING",
"ADAPTER_NOT_AVAILABLE",
NULL,
"MSG_ID_ERROR",
"CORREL_ID_ERROR",
"FILE_SYSTEM_ERROR",
"NO_MSG_LOCKED",
"SOAP_DOTNET_ERROR",         /* 2210 - 2212 new for MQ 6.0 */
"SOAP_AXIS_ERROR",
"SOAP_URL_ERROR",
NULL,
NULL,
NULL,
"FILE_NOT_AUDITED",
"CONNECTION_NOT_AUTHORIZED",
"MSG_TOO_BIG_FOR_CHANNEL",
"CALL_IN_PROGRESS",
"RMH_ERROR",
NULL,
"Q_MGR_ACTIVE",
"Q_MGR_NOT_ACTIVE",
"Q_DEPTH_HIGH",
"Q_DEPTH_LOW",
"Q_SERVICE_INTERVAL_HIGH",
"Q_SERVICE_INTERVAL_OK",
"RFH_HEADER_FIELD_ERROR",    /* 2228 - 2229 new for MQ 6.0 */
"RAS_PROPERTY_ERROR",
NULL,
NULL,
"UNIT_OF_WORK_NOT_STARTED",  /* 2232, new for MQ 5.1 */
"CHANNEL_AUTO_DEF_OK",
"CHANNEL_AUTO_DEF_ERROR",
"CFH_ERROR",
"CFIL_ERROR",
"CFIN_ERROR",
"CFSL_ERROR",
"CFST_ERROR",
NULL,
"INCOMPLETE_GROUP",
"INCOMPLETE_MSG",
"INCONSISTENT_CCSIDS",
"INCONSISTENT_ENCODINGS",
"INCONSISTENT_UOW",
"INVALID_MSG_UNDER_CURSOR",
"MATCH_OPTIONS_ERROR",
"MDE_ERROR",
"MSG_FLAGS_ERROR",
"MSG_SEQ_NUMBER_ERROR",
"OFFSET_ERROR",
"ORIGINAL_LENGTH_ERROR",
"SEGMENT_LENGTH_ZERO",
NULL,
"UOW_NOT_AVAILABLE",
"WRONG_GMO_VERSION",
"WRONG_MD_VERSION",
"GROUP_ID_ERROR",
"INCONSISTENT_BROWSE",
"XQH_ERROR",
"SRC_ENV_ERROR",
"SRC_NAME_ERROR",
"DEST_ENV_ERROR",
"DEST_NAME_ERROR",
"TM_ERROR",
"CLUSTER_EXIT_ERROR",        /* 2266 - 2279, new for MQ 5.1 */
"CLUSTER_EXIT_LOAD_ERROR",
"CLUSTER_PUT_INHIBITED",
"CLUSTER_RESOURCE_ERROR",
"NO_DESTINATIONS_AVAILABLE",
"CONN_TAG_IN_USE",
"PARTIALLY_CONVERTED",
"CONNECTION_ERROR",
"OPTION_ENVIRONMENT_ERROR",
NULL,
NULL,
"CD_ERROR",
"CLIENT_CONN_ERROR",
"CHANNEL_STOPPED_BY_USER",
"HCONFIG_ERROR",
"FUNCTION_ERROR",
"CHANNEL_STARTED",
"CHANNEL_STOPPED",
"CHANNEL_CONV_ERROR",
"SERVICE_NOT_AVAILABLE",
"INITIALIZATION_FAILED",
"TERMINATION_FAILED",
"UNKNOWN_Q_NAME",
"SERVICE_ERROR",
"Q_ALREADY_EXISTS",
"USER_ID_NOT_AVAILABLE",
"UNKNOWN_ENTITY",
"UNKNOWN_AUTH_ENTITY",
"UNKNOWN_REF_OBJECT",
"CHANNEL_ACTIVATED",
"CHANNEL_NOT_ACTIVATED",     /* 2296, last in MQ 2.2 */
"UOW_CANCELED",
"FUNCTION_NOT_SUPPORTED",
"SELECTOR_TYPE_ERROR",
"COMMAND_TYPE_ERROR",        /* 2300 */
"MULTIPLE_INSTANCE_ERROR",
"SYSTEM_ITEM_NOT_ALTERABLE",
"BAG_CONVERSION_ERROR",
"SELECTOR_OUT_OF_RANGE",
"SELECTOR_NOT_UNIQUE",
"INDEX_NOT_PRESENT",
"STRING_ERROR",
"ENCODING_NOT_SUPPORTED",
"SELECTOR_NOT_PRESENT",
"OUT_SELECTOR_ERROR",
"STRING_TRUNCATED",
"SELECTOR_WRONG_TYPE",
"INCONSISTENT_ITEM_TYPE",
"INDEX_ERROR",
"SYSTEM_BAG_NOT_ALTERABLE",
"ITEM_COUNT_ERROR",
"FORMAT_NOT_SUPPORTED",
"SELECTOR_NOT_SUPPORTED",
"ITEM_VALUE_ERROR",
"HBAG_ERROR",
"PARAMETER_MISSING",
"CMD_SERVER_NOT_AVAILABLE",
"STRING_LENGTH_ERROR",
"INQUIRY_COMMAND_ERROR",
"NESTED_BAG_NOT_SUPPORTED",
"BAG_WRONG_TYPE",
"ITEM_TYPE_ERROR",
"SYSTEM_BAG_NOT_DELETABLE",
"SYSTEM_ITEM_NOT_DELETABLE",
"CODED_CHAR_SET_ID_ERROR",
"MSG_TOKEN_ERROR",
"MISSING_WIH",
"WIH_ERROR",
"RFH_ERROR",
"RFH_STRING_ERROR",          /* 2335, last in MQ 5.0 */
"RFH_COMMAND_ERROR",
"RFH_PARM_ERROR",
"RFH_DUPLICATE_PARM",
"RFH_PARM_MISSING",          /* 2339, last in MQ 5.1, see also 2187-2190, 2232, 2266-2279 */
"CHAR_CONVERSION_ERROR",
"UCS2_CONVERSION_ERROR",
"DB2_NOT_AVAILABLE",
"OBJECT_NOT_UNIQUE",
"CONN_TAG_NOT_RELEASED",
"CF_NOT_AVAILABLE",
"CF_STRUC_IN_USE",
"CF_STRUC_LIST_HDR_IN_USE",
"CF_STRUC_AUTH_FAILED",
"CF_STRUC_ERROR",
"CONN_TAG_NOT_USABLE",
"GLOBAL_UOW_CONFLICT",
"LOCAL_UOW_CONFLICT",
"HANDLE_IN_USE_FOR_UOW",
"UOW_ENLISTMENT_ERROR",
"UOW_MIX_NOT_SUPPORTED",
"WXP_ERROR",
"CURRENT_RECORD_ERROR",
"NEXT_OFFSET_ERROR",
"NO_RECORD_AVAILABLE",
"OBJECT_LEVEL_INCOMPATIBLE",
"NEXT_RECORD_ERROR",         /* 2361, last in MQ 5.2 */
"BACKOUT_THRESHOLD_REACHED",
"MSG_NOT_MATCHED",
"JMS_FORMAT_ERROR",
"SEGMENTS_NOT_SUPPORTED",
"WRONG_CF_LEVEL",
"CONFIG_CREATE_OBJECT",
"CONFIG_CHANGE_OBJECT",
"CONFIG_DELETE_OBJECT",
"CONFIG_REFRESH_OBJECT",
"CHANNEL_SSL_ERROR",
"PARTICIPANT_NOT_DEFINED",   /* 2372 new in MQ 7.0 */
"CF_STRUC_FAILED",
"API_EXIT_ERROR",
"API_EXIT_INIT_ERROR",
"API_EXIT_TERM_ERROR",
"EXIT_REASON_ERROR",
"RESERVED_VALUE_ERROR",
"NO_DATA_AVAILABLE",
"SCO_ERROR",
"KEY_REPOSITORY_ERROR",
"CRYPTO_HARDWARE_ERROR",
"AUTH_INFO_REC_COUNT_ERROR",
"AUTH_INFO_REC_ERROR",
"AIR_ERROR",
"AUTH_INFO_TYPE_ERROR",
"AUTH_INFO_CONN_NAME_ERROR",
"LDAP_USER_NAME_ERROR",
"LDAP_USER_NAME_LENGTH_ERR",
"LDAP_PASSWORD_ERROR",
"SSL_ALREADY_INITIALIZED",
"SSL_CONFIG_ERROR",
"SSL_INITIALIZATION_ERROR",
"Q_INDEX_TYPE_ERROR",
"CFBS_ERROR",                /* 2395 new for MQ 6.0 */
"SSL_NOT_ALLOWED",
"JSSE_ERROR",
"SSL_PEER_NAME_MISMATCH",
"SSL_PEER_NAME_ERROR",
"UNSUPPORTED_CIPHER_SUITE",  /* 2400 */
"SSL_CERTIFICATE_REVOKED",
"SSL_CERT_STORE_ERROR",      /* 2402, last in MQ 5.3 */
NULL,
NULL,
NULL,
"CLIENT_EXIT_LOAD_ERROR",
"CLIENT_EXIT_ERROR",
"UOW_COMMITTED",             /* 2408 new in MQ 7.0 */
"SSL_KEY_RESET_ERROR",
"UNKNOWN_COMPONENT_NAME",
"LOGGER_STATUS",
"COMMAND_MQSC",
"COMMAND_PCF",
"CFIF_ERROR",
"CFSF_ERROR",
"CFGR_ERROR",
"MSG_NOT_ALLOWED_IN_GROUP",
"FILTER_OPERATOR_ERROR",
"NESTED_SELECTOR_ERROR",
"EPH_ERROR",
"RFH_FORMAT_ERROR",
"CFBF_ERROR",
"CLIENT_CHANNEL_CONFLICT",   /* 2423, last in MQ 6.0 */
"SD_ERROR",
"TOPIC_STRING_ERROR",
"STS_ERROR",
NULL,
"NO_SUBSCRIPTION",
"SUBSCRIPTION_IN_USE",
"STAT_TYPE_ERROR",
"SUB_USER_DATA_ERROR",
"SUB_ALREADY_EXISTS",
NULL,
"IDENTITY_MISMATCH",
"ALTER_SUB_ERROR",
"DURABILITY_NOT_ALLOWED",
"NO_RETAINED_MSG",
"SRO_ERROR",
NULL,
"SUB_NAME_ERROR",
"OBJECT_STRING_ERROR",
"PROPERTY_NAME_ERROR",
"SEGMENTATION_NOT_ALLOWED",
"CBD_ERROR",
"CTLO_ERROR",
"NO_CALLBACKS_ACTIVE",
NULL,
"CALLBACK_NOT_REGISTERED",
NULL,
NULL,                        /* 2450 */
NULL,
NULL,
NULL,
NULL,
NULL,
NULL,
"OPTIONS_CHANGED",
"READ_AHEAD_MSGS",
"SELECTOR_SYNTAX_ERROR",
"HMSG_ERROR",
"CMHO_ERROR",
"DMHO_ERROR",
"SMPO_ERROR",
"IMPO_ERROR",
"PROPERTY_NAME_TOO_BIG",
"PROP_VALUE_NOT_CONVERTED",
"PROP_TYPE_NOT_SUPPORTED",
NULL,
"PROPERTY_VALUE_TOO_BIG",
"PROP_CONV_NOT_SUPPORTED",
"PROPERTY_NOT_AVAILABLE",
"PROP_NUMBER_FORMAT_ERROR",
"PROPERTY_TYPE_ERROR",
NULL,
NULL,
NULL,
NULL,
"PROPERTIES_TOO_BIG",
"PUT_NOT_RETAINED",
"ALIAS_TARGTYPE_CHANGED",
"DMPO_ERROR",
"PD_ERROR",
"CALLBACK_TYPE_ERROR",
"CBD_OPTIONS_ERROR",
"MAX_MSG_LENGTH_ERROR",
"CALLBACK_ROUTINE_ERROR",
"CALLBACK_LINK_ERROR",
"OPERATION_ERROR",
"BMHO_ERROR",
"UNSUPPORTED_PROPERTY",
NULL,
"PROP_NAME_NOT_CONVERTED",
NULL,
"GET_ENABLED",
"MODULE_NOT_FOUND",
"MODULE_INVALID",
"MODULE_ENTRY_NOT_FOUND",
"MIXED_CONTENT_NOT_ALLOWED",
"MSG_HANDLE_IN_USE",
"HCONN_ASYNC_ACTIVE",        /* 2500 */
"MHBO_ERROR",
"PUBLICATION_FAILURE",
"SUB_INHIBITED",
"SELECTOR_ALWAYS_FALSE",
NULL,
NULL,
"XEPO_ERROR",
NULL,
"DURABILITY_NOT_ALTERABLE",
"TOPIC_NOT_ALTERABLE",
NULL,
"SUBLEVEL_NOT_ALTERABLE",
"PROPERTY_NAME_LENGTH_ERR",
"DUPLICATE_GROUP_SUB",
"GROUPING_NOT_ALTERABLE",
"SELECTOR_INVALID_FOR_TYPE",
"HOBJ_QUIESCED",
"HOBJ_QUIESCED_NO_MSGS",
"SELECTION_STRING_ERROR",
"RES_OBJECT_STRING_ERROR",
"CONNECTION_SUSPENDED",
"INVALID_DESTINATION",
"INVALID_SUBSCRIPTION",
"SELECTOR_NOT_ALTERABLE",
"RETAINED_MSG_Q_ERROR",
"RETAINED_NOT_DELIVERED",
"RFH_RESTRICTED_FORMAT_ERR",
"CONNECTION_STOPPED",
"ASYNC_UOW_CONFLICT",
"ASYNC_XA_CONFLICT",
"PUBSUB_INHIBITED",
"MSG_HANDLE_COPY_FAILURE",
"DEST_CLASS_NOT_ALTERABLE",
"OPERATION_NOT_ALLOWED",
"ACTION_ERROR",
NULL,
"CHANNEL_NOT_AVAILABLE",
"HOST_NOT_AVAILABLE",
"CHANNEL_CONFIG_ERROR",
"UNKNOWN_CHANNEL_NAME",
"LOOPING_PUBLICATION",
"ALREADY_JOINED"             /* 2542, last in MQ 7.0 */
 };

static char *RcLits6[] = 
 {
"REOPEN_EXCL_INPUT_ERROR",   /* 6100 */
"REOPEN_INQUIRE_ERROR", 
"REOPEN_SAVED_CONTEXT_ERR",
"REOPEN_TEMPORARY_Q_ERROR",
"ATTRIBUTE_LOCKED",
"CURSOR_NOT_VALID",
"ENCODING_ERROR",
"STRUC_ID_ERROR",
"NULL_POINTER",
"NO_CONNECTION_REFERENCE",
"NO_BUFFER",
"BINARY_DATA_LENGTH_ERROR",
"BUFFER_NOT_AUTOMATIC",
"INSUFFICIENT_BUFFER",
"INSUFFICIENT_DATA",
"DATA_TRUNCATED",
"ZERO_LENGTH",
"NEGATIVE_LENGTH",
"NEGATIVE_OFFSET",
"INCONSISTENT_FORMAT",
"INCONSISTENT_OBJECT_STATE",
"CONTEXT_OBJECT_NOT_VALID",
"CONTEXT_OPEN_ERROR",
"STRUC_LENGTH_ERROR",
"NOT_CONNECTED",
"NOT_OPEN",
"DISTRIBUTION_LIST_EMPTY",
"INCONSISTENT_OPEN_OPTIONS",
"WRONG_VERSION",
"REFERENCE_ERROR"            /* 6129, last in MQ 5.2, 5.3, 6.0, 7.0 */ 
 };


/***************************************************************
 *
 * Function: MQCompCodeMsg
 *
 * Purpose:  Convert a WebSphere MQ Completion Code to the
 *           symbolic name in a C string.  Returns
 *           "MQCC_nnnn_UNDEFINED" for undefined completion codes.
 *
 * Arguments:
 *  Name    Direction  Type     Purpose
 *
 *  Cc      IN         MQLONG   Completion Code
 *  RetStr  IN         * char   Buffer to contain symbolic name,
 *                              size must be at least
 *                              MQCM_MINRETSTRSIZE chars
 *  return  OUT        * char   Symbolic name of the completion
 *                              code in null terminated string
 *
 ***************************************************************/

char *MQCompCodeMsg( MQLONG Cc, char *RetStr )
{
 if( Cc >= MQCM_CCMIN && Cc <= MQCM_CCMAX ) 
   sprintf( RetStr, "MQCC_%s", CcLits[ Cc - MQCM_CCMIN ] );
 else
   sprintf( RetStr, "MQCC_%d_UNDEFINED", Cc );

 return( RetStr );
}


/***************************************************************
 *
 * Function: MQReasonCodeMsg
 *
 * Purpose:  Convert a WebSphere MQ Reason Code to the
 *           symbolic name in a C string.  Returns
 *           "MQRC_nnnn_UNDEFINED" for undefined reason codes.
 *
 * Arguments:
 *  Name    Direction  Type     Purpose
 *
 *  Rc      IN         MQLONG   Reason Code
 *  RetStr  IN         * char   Buffer to contain symbolic name,
 *                              size must be at least
 *                              MQCM_MINRETSTRSIZE chars
 *  return  OUT        * char   Symbolic name of the reason
 *                              code in null terminated string
 *
 ***************************************************************/

char *MQReasonCodeMsg( MQLONG Rc, char *RetStr )
{
 if( Rc == MQRC_NONE )
   strcpy( RetStr, "MQRC_NONE" );

 else if( Rc >= MQCM_RCMIN && Rc <= MQCM_RCMAX ) 
   {
     if( RcLits2[ Rc - MQCM_RCMIN ] != NULL )
       sprintf( RetStr, "MQRC_%s", RcLits2[ Rc - MQCM_RCMIN ] );
     else
       sprintf( RetStr, "MQRC_%d_UNDEFINED", Rc );
   }

 else if( (Rc >= MQCM_RC6MIN) && (Rc <= MQCM_RC6MAX) ) 
   {
     if( RcLits6[ Rc - MQCM_RC6MIN ] != NULL )
       sprintf( RetStr, "MQRC_%s", RcLits6[ Rc - MQCM_RC6MIN ] );
     else
       sprintf( RetStr, "MQRC_%d_UNDEFINED", Rc );
   }

#ifdef MQRC_APPL_FIRST
 else if( Rc == MQRC_APPL_FIRST ) 
   strcpy( RetStr, "MQRC_APPL_FIRST" );
 else if( Rc == MQRC_APPL_LAST ) 
   strcpy( RetStr, "MQRC_APPL_LAST" );
 else if( Rc > MQRC_APPL_FIRST && Rc < MQRC_APPL_LAST ) 
   sprintf( RetStr, "MQRC_APPL_%d", Rc );
#endif

 else
   sprintf( RetStr, "MQRC_%d_UNDEFINED", Rc );

 return( RetStr );
}

