/*
 * IBM SupportPac MA0K
 * WebSphere MQ Completion Code and Reason Code analysis programs V1.4
 * http://www.ibm.com/support/docview.wss?uid=swg24000652
 *
 * MQMCH - WebSphere MQ Messages and Codes Helper program
 *
 * "mqmqch.c" main program module in C
 *
 * Source file: \\IBM-L3V1408\WMQ-PROJ\MA0K-v1.4-Build\_Source\mqmch-18.c
 *
 * Environments: Windows 32 bit, Unix (AIX, HP-UX, Solaris, Linux)
 *
 * This program accepts a MQI Completion Code or Reason Code
 * numeric value and displays the corresponding symbol name(s).
 * A part of the symbolic name for a code is also accepted and
 * partial matches are displayed.
 *
 * Revision History
 * ================
 * Version  Date         Author    Changes
 *     1.1  14 Feb 2000  Glenn     Initial version for MQSeries 5.1
 *                       Baddeley
 *       2  03 Mar 2000  GB        Add '-make' command switch,
 *                                 Add TellDescription()
 *       3  07 Mar 2000  GB        TellDescription uses $_ variable for file name
 *       4  31 Mar 2000  GB        Add '-palm'
 *       5  03 Apr 2000  GB        Support Siemens Sinix
 *       6  01 May 2000  GB        Check if MQRC_APPL defined before using
 *       7  22 Nov 2000  GB        Use new MQ message functions calling seq
 *       8  09 Feb 2001  GB        Modify MakeDescriptions to read PDF
 *                                 extracted text, Add FixCommas function,
 *                                 Add PrintDescription function
 *       9  02 Mar 2001  GB        Add doc info stage to MakeDescriptions,
 *                                 Change -make to -text
 *      10  19 Oct 2001  GB        MakeDescriptions not to use MQReasonCodeMsg 
 *                                 and MQCompCodeMsg to look up symbolic names
 *                                 as 2192 has to sym names and two descs,
 *                                 MakeDescription includes sym name in output
 *                                 section name for DI_VERSION 1,
 *                                 Major changes to search methods
 *      11  15 Jan 2002  GB        In SearchSymbols, dont memchr with null ptr
 *      12  25 Feb 2002  GB        Generate correct numeric codes for Palm data
 *      13  29 Jul 2002  GB        Rebrand MQSeries to WebSphere MQ 5.3
 *      14  30 Jan 2003  GB        In MakeDescriptions, ignore leading '|',
 *                                 handle 'This reason code', add StrBufFix,
 *                                 strip all trailing spaces in one place
 *      15   4 Feb 2003  GB        Support AIX
 *      16  28 Jan 2004  GB        Change copyright owner from IBM GSA to IBM Australia,
 *                                 Support Linux (Red Hat 7.3 on x86)
 *      17  25 Oct 2005  GB        Update for WMQ 6.0, Remove support for Sinix,
 *                                 Support Windows 32 bit command prompt
 *      18  25 Sep 2008  GB        Update for WMQ 7.0 GA
 *                                 Revise copyright and owner information
 *                                 Remove functions MakeDescriptions, FixCommas
 *                                 and remove -text -palm as these functions are
 *                                 now done in Windows mqmch VB program
 */
#define SRCID "@(#)MQMCH.C VER=1.18 DATE=25-Sep-2008"

/*  <N_OCO_COPYRIGHT>
 *  Licensed Materials - Property of IBM
 *  (c) Copyright IBM Corp. 2008 All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *  <NOC_COPYRIGHT>                                                   */
#define OWNID "@(#)Licensed Materials - Property of IBM"
#define CPRID "@(#)Copyright 2008 Corp. All Rights Reserved"

#ifdef _WIN32
  #include <windows.h>
  #include <stdio.h>
#else  /* Unix */
  #include <stdlib.h>
  #include <string.h>
  #include <ctype.h>
  #include <stdio.h>
  #include <errno.h>
#endif

/* Define MQLONG */
#include <cmqc.h>

/* Defines */
#define DI_VERSION 1
#define MAXCMDENT 2
#define BLKBUFFLEN 32768
#define MAXDESCLEN 4096
#define MAXHEADERLEN 48
#define MINHEADERLEN 8
#define MAXCHARSPERLINE 78

/* Prototypes of local functions */
int OpenDescriptions( char *DescFileName );
int SearchSectionHeader( char *HeaderStr );
int SearchSymbols( char *SymbolStr );
void PrintDescription( char *pDesc );
char *MemSearch( char *pMem, int MemLen, char *pMatch, int MatchLen );

/* General data */
static char SrcId[] = SRCID, CprId[] = CPRID, OwnId[] = OWNID;
static char *pBlkBuff = NULL;
static int SwDebug = 0, SwDescription = 0, DescFileLen = 0, MatchCnt = 0;
static FILE *fBin = NULL;
 
/****************/
/* Main Program */
/****************/
int main(int ArgC, char *ArgP[])
{
MQLONG CodeNum;
char *ChP, *pArgLit, *pProgFile, *CmdEnt[MAXCMDENT], ArgLit[64], SectId[64];
int ArgIdx, CmdEntCnt=0;
int SwVersion=0;
int Idx, HeadFlag, MatchCode, MatchSect, ScIdx, ExitCode=0;

#ifdef MQRC_APPL_FIRST
int ScRc[] = { MQRC_NONE, MQRC_APPL_FIRST, MQRC_APPL_LAST };
#else
int ScRc[] = { MQRC_NONE };
#endif

#ifdef _WIN32
char ProgName[MAX_PATH];
#endif

  /* File name of program binary */
#ifdef _WIN32
  GetModuleFileName( NULL, pProgFile = ProgName, MAX_PATH );
#else
  pProgFile = getenv( "_" );
#endif

  /*  Parse command line */
  if( ArgC > 1 )
    for( ArgIdx=1 ; ArgIdx<ArgC ; ArgIdx++ )
      {
        if( strcmp( ArgP[ArgIdx], "-d" ) == 0 )
          SwDescription = 1;
        else if( strcmp( ArgP[ArgIdx], "-v" ) == 0 )
          SwVersion = 1;
        else if( strcmp( ArgP[ArgIdx], "-debug" ) == 0 )
          SwDebug = 1;
        else if( CmdEntCnt < MAXCMDENT )
          CmdEnt[CmdEntCnt++] = ArgP[ArgIdx];
      }

  if( CmdEntCnt == 0 )
    if( SwVersion )  /* Version from document information */
     {
      if( (ExitCode = OpenDescriptions( pProgFile )) != 0 )
        exit( ExitCode );
      SwDescription = 1;
      sprintf( SectId, "[DI%d,Version]", DI_VERSION );
      exit( SearchSectionHeader( SectId ) );
     }
    else
     {
      printf( "WebSphere MQ Messages and Codes Helper program\n" );
      printf( "%s\n%s, %s\n", SrcId+4, CprId+4, OwnId+4 );
      printf( "\nCommand line formats are:\nmqmch Code [-d]\nmqmch -v\n\n" );
      printf( "Code may be a decimal number or a hexadecimal number (prefix with X)\n" );
      printf( "which is the value of a MQI Completion Code or Reason Code.\n\n" );
      printf( "Code may also be part of the symbolic name for a code (case is insignificant).\n\n" );
      printf( "-d        displays a full description of a Code\n" );
      printf( "-v        displays the messages version information\n\n" );
      printf( "eg. mqmch 2035 -d\n    mqmch NOT_AUTH\n" );
      printf( "    mqmch MQRC\n    mqmch x7f2\n" );
      exit( 1 );
     }

  /* Convert first argument to upper case */
  for( ChP = CmdEnt[0], pArgLit = ArgLit ; *ChP ; *pArgLit++ = toupper( *ChP++ ) );
  *pArgLit = '\0';

  if( strspn( ArgLit, "0123456789-" ) != 0 || toupper( ArgLit[0] ) == 'X' )
    { /* Numeric code */
      if( toupper( ArgLit[0] ) == 'X' )
        sscanf( &ArgLit[1], "%x", &CodeNum );   /* hex */
      else
        CodeNum = atoi( ArgLit );   /* decimal */

#ifdef MQRC_APPL_FIRST
      /* Check if within application code number range */
      if( CodeNum > MQRC_APPL_FIRST && CodeNum < MQRC_APPL_LAST )
        {
          printf( "Reason Code %d x%X\nMQRC_APPL_FIRST + %d\n",
                  CodeNum, CodeNum, CodeNum - MQRC_APPL_FIRST );
          printf( "Refer to MQRC_APPL_FIRST and MQRC_APPL_LAST for details.\n" );
          MatchCnt++;
        }
      else
#endif
      {
        if( (ExitCode = OpenDescriptions( pProgFile )) != 0 )
          exit( ExitCode );
        sprintf( SectId, "[%d,", CodeNum );
        ExitCode = SearchSectionHeader( SectId );
      }

      /* Summary */
      if( MatchCnt == 0 )
        printf( "%d x%X is not a known Completion Code or Reason Code.\n", CodeNum, CodeNum );
      else if( ! SwDescription )
        if( MatchCnt == 1 )
          printf( "\nOne match found on numeric codes. Use -d to view full description.\n" );
        else
          printf( "\n%d matches found on numeric codes. Use -d to view full description.\n", MatchCnt );
    }
  else if( strlen( ArgLit ) >= 3 )
    {
      if( (ExitCode = OpenDescriptions( pProgFile )) != 0 )
        exit( ExitCode );
      ExitCode = SearchSymbols( ArgLit );

      /* Summary */
      if( MatchCnt == 0 )
        printf( "'%s' is not part of a symbolic name for a Completion Code or Reason Code.\n", ArgLit );
      else if( ! SwDescription )
        if( MatchCnt == 1 )
          printf( "\nOne match found on symbolic names. Use -d to view full description.\n" );
        else
          printf( "\n%d matches found on symbolic names. Use -d to view full description.\n", MatchCnt );
    }
  else
    printf( "'%s' part of symbolic name must be at least 3 characters.\n", ArgLit );

  if( fBin != NULL )
    if( fclose( fBin ) != 0 )
      fprintf( stderr, "Error: Can't close descriptions file: (%d) %s\n",
               errno, strerror( errno ) );

  if( pBlkBuff != NULL )
    free( pBlkBuff );

  exit( ExitCode );
}

/**************************************************************************/
/* Open a file name as the descriptions database and allocate a buffer.   */
/**************************************************************************/

int OpenDescriptions( char *DescFileName )
{
  if( (fBin = fopen( DescFileName, "rb" )) == NULL )
    {
      fprintf( stderr, "Error: Can't open file '%s' to read: (%d) %s\n",
               DescFileName, errno, strerror( errno ) );
      return 2;
    }

  if( fseek( fBin, 0, SEEK_END ) != 0 )
    {
      fprintf( stderr, "Error: Can't seek to end in file '%s'\n", DescFileName );
      fclose( fBin );
      return 6;
    }

  if( (DescFileLen = ftell( fBin )) == -1 )
    {
      fprintf( stderr, "Error: Can't get length of file '%s'\n", DescFileName );
      fclose( fBin );
      return 7;
    }
  if( (pBlkBuff = malloc( BLKBUFFLEN )) == NULL )
    {
      fprintf( stderr, "Error: Unable to allocate %d bytes of memory\n", BLKBUFFLEN );
      fclose( fBin );
      return 1;
    }

  return 0;
}

/**************************************************************************/
/* Given a section header prefix (starts with '['), search the            */
/* descriptions data base for matches and then optionally display the     */
/* following descriptive text.                                            */
/**************************************************************************/

int SearchSectionHeader( char *HeaderStr )
{
char *pFindPrev, *pFind;
char *pHdrField1, *pHdrField2, *pHdrEnd;
char pDescBuff[MAXDESCLEN], *pDescEnd;
int rc=0, FindLen, Searching=1, SeekAddr=0, SeekDescAddr, HeaderLen, CodeNum;

  HeaderLen = strlen( HeaderStr );
  while( Searching && (SeekAddr < DescFileLen) )
    {
      /* Read a big block */
      if( fseek( fBin, SeekAddr, SEEK_SET ) != 0 )
        {
          fprintf( stderr, "Error: Can't seek to %d in descriptions file '%s'\n", SeekAddr );
          rc = 3;
          break;
        }
      fread( pBlkBuff, BLKBUFFLEN, 1, fBin );
      if( ferror( fBin ) != 0 )
        {
          fprintf( stderr, "Error: Can't read descriptions file: (%d) %s\n",
                   errno, strerror( errno ) );
          rc = 4;
          break;
        }

      /* Search the block for matching section headers */
      pFindPrev = pBlkBuff;
      FindLen = BLKBUFFLEN;
      while( (pFind = MemSearch( pFindPrev, FindLen, HeaderStr, HeaderLen )) != NULL )
          {
            /* Found match, identify fields in the header */
            MatchCnt++;
            pHdrField1 = pFind + 1;
            pHdrField2 = strchr( pHdrField1, ',' );
            *pHdrField2++ = '\0';
            pHdrEnd = strchr( pHdrField2, ']' );
            *pHdrEnd = '\0';

            /* Display the match */
            if( MatchCnt > 1 && SwDescription )
              printf( "\n" );

            if( *(pHdrField2+2) == 'C' )
              {
                CodeNum = atoi( pHdrField1 );
                printf( "Completion Code %d x%X %s\n", CodeNum, CodeNum, pHdrField2 );
              }
            else if( *(pHdrField2+2) == 'R' )
              {
                CodeNum = atoi( pHdrField1 );
                printf( "Reason Code %d x%X %s\n", CodeNum, CodeNum, pHdrField2 );
              }

            if( SwDescription )
              {
                /* Seek and read the full descriptive text */
                SeekDescAddr = SeekAddr + pHdrEnd + 2 - pBlkBuff;
                if( fseek( fBin, SeekDescAddr, SEEK_SET ) != 0 )
                  {
                    fprintf( stderr, "Error: Can't seek to %d in descriptions file\n",
                             SeekDescAddr );
                    SeekAddr = DescFileLen;
                    rc = 8;
                    break;
                  }
                fread( pDescBuff, MAXDESCLEN, 1, fBin );
                if( ferror( fBin ) != 0 )
                  {
                    fprintf( stderr, "Error: Can't read descriptions file: (%d) %s\n",
                             errno, strerror( errno ) );
                    SeekAddr = DescFileLen;
                    rc = 9;
                    break;
                  }

                /* Prepare descriptive text for display */
               if( (pDescEnd = memchr( pDescBuff, ']', MAXDESCLEN )) == NULL )
                  {
                    fprintf( stderr, "Error: Can't find end of description: Hdr='%s' MaxLen=%d\n",
                             HeaderStr, MAXDESCLEN );
                    rc = 10;
                    break;
                  }
                *pDescEnd = '\0';
                PrintDescription( pDescBuff );
              };

            /* Look for another match in the block */
            FindLen -= pFind - pFindPrev + 1;
            pFindPrev = pFind + 1;
          }

      /* Read the next block, with a little bit of overlap */
      SeekAddr += BLKBUFFLEN - MAXHEADERLEN;
    }

  return rc;
}

/**************************************************************************/
/* Given a (partial) symbolic name, search the descriptions data base for */
/* matches and optionally display the following descriptive text.         */
/**************************************************************************/

int SearchSymbols( char *SymbolStr )
{
char *pFindPrev, *pFind;
char *pHdrBeg, *pHdrEnd, *pHdrField1, *pHdrField2;
char pDescBuff[MAXDESCLEN], *pDescEnd;
int rc=0, FindLen, Searching=1, SeekAddr=0, SeekDescAddr, SymbolLen, CodeNum;

  SymbolLen = strlen( SymbolStr );
  while( Searching && (SeekAddr < DescFileLen) )
    {
      /* Read a big block */
      if( fseek( fBin, SeekAddr, SEEK_SET ) != 0 )
        {
          fprintf( stderr, "Error: Can't seek to %d in descriptions file '%s'\n", SeekAddr );
          rc = 3;
          break;
        }
      fread( pBlkBuff, BLKBUFFLEN, 1, fBin );
      if( ferror( fBin ) != 0 )
        {
          fprintf( stderr, "Error: Can't read descriptions file: (%d) %s\n",
                   errno, strerror( errno ) );
          rc = 4;
          break;
        }

      /* Search the block for symbolic string */
      pFindPrev = pBlkBuff;
      FindLen = BLKBUFFLEN;
      while( (pFind = MemSearch( pFindPrev, FindLen, SymbolStr, SymbolLen )) != NULL )
        {
          /* Found match, check that its in a section header */
          if( (pHdrBeg = memchr( pFind - MAXHEADERLEN, '[', MAXHEADERLEN )) != NULL )
            pHdrEnd = memchr( pHdrBeg, ']', MAXHEADERLEN );
          else
            pHdrEnd = NULL;

          if( pHdrBeg != NULL                   &&
              pHdrBeg >= pBlkBuff               &&
              pHdrEnd != NULL                   &&
              pHdrEnd - pHdrBeg <= MAXHEADERLEN &&
              pHdrEnd - pHdrBeg >= MINHEADERLEN )
           {
            /* Identify fields in the header */
            MatchCnt++;
            pHdrField1 = pHdrBeg + 1;
            pHdrField2 = strchr( pHdrField1, ',' );
            *pHdrField2++ = '\0';
            *pHdrEnd = '\0';

            /* Display the match */
            if( MatchCnt > 1 && SwDescription )
              printf( "\n" );

            if( *(pHdrField2+2) == 'C' )
              {
                CodeNum = atoi( pHdrField1 );
                printf( "Completion Code %d x%X %s\n", CodeNum, CodeNum, pHdrField2 );
              }
            else if( *(pHdrField2+2) == 'R' )
              {
                CodeNum = atoi( pHdrField1 );
                printf( "Reason Code %d x%X %s\n", CodeNum, CodeNum, pHdrField2 );
              }

            if( SwDescription )
              {
                /* Seek and read the full descriptive text */
                SeekDescAddr = SeekAddr + pHdrEnd + 2 - pBlkBuff;
                if( fseek( fBin, SeekDescAddr, SEEK_SET ) != 0 )
                  {
                    fprintf( stderr, "Error: Can't seek to %d in descriptions file\n",
                             SeekDescAddr );
                    SeekAddr = DescFileLen;
                    rc = 8;
                    break;
                  }
                fread( pDescBuff, MAXDESCLEN, 1, fBin );
                if( ferror( fBin ) != 0 )
                  {
                    fprintf( stderr, "Error: Can't read descriptions file: (%d) %s\n",
                             errno, strerror( errno ) );
                    SeekAddr = DescFileLen;
                    rc = 9;
                    break;
                  }

                /* Prepare descriptive text for display */
               if( (pDescEnd = memchr( pDescBuff, ']', MAXDESCLEN )) == NULL )
                  {
                    fprintf( stderr, "Error: Can't find end of description: Hdr='%s' MaxLen=%d\n",
                             pHdrBeg, MAXDESCLEN );
                    rc = 10;
                    break;
                  }
                *pDescEnd = '\0';
                PrintDescription( pDescBuff );
              }
            }

          /* Look for another match in the block */
          FindLen -= pFind - pFindPrev + 1;
          pFindPrev = pFind + 1;
        }

      /* Read the next block, with a little bit of overlap */
      SeekAddr += BLKBUFFLEN - MAXHEADERLEN;
    }

  return rc;
}

/**************************************************************************/
/* Given a null terminated string with embedded new-lines, print it on    */
/* standard-out with a limited line length, doing line breaks at the      */
/* previous space character.                                              */
/**************************************************************************/

void PrintDescription( char *pDesc )
{
int DescOff=-1, PrOff=0, SpOff=-1, MvOff;
char cDesc, PrBuff[MAXCHARSPERLINE+2];

  while( (cDesc = *pDesc++) != '\0' )
    if( cDesc == '\n' )
      { /* End of line, print everything in buffer */
        PrBuff[ PrOff ] = '\0';
        printf( "%s\n", PrBuff );
        PrOff = 0;
        SpOff = -1;
      }
    else
      {
        if( cDesc == ' ' ) 
          SpOff = PrOff;  /* Remember the most recent space */
        if( PrOff == MAXCHARSPERLINE )
          {
            if( SpOff == PrOff )
              { /* The maximum line length is at a convenient space */
                PrBuff[ PrOff ] = '\0';
                printf( "%s\n", PrBuff );
                PrOff = 0;
                SpOff = -1;
              }
            else
              { /* Print everything up to the previous space */
                PrBuff[ SpOff++ ] = '\0';
                printf( "%s\n", PrBuff );
                PrBuff[ PrOff++ ] = cDesc;
                /* Move the word after the previous space to the
                   start of the buffer and continue from there */
                MvOff = 0;
                while( SpOff <= PrOff )
                  PrBuff[ MvOff++ ] = PrBuff[ SpOff++ ];
                PrOff = MvOff - 1;
                SpOff = -1;
              }
          }
        else  /* Add the character to the print buffer */
          PrBuff[ PrOff++ ] = cDesc;
      }
  /* PrOff will be 0 on return because all descriptions end in \n\0 */
}

/**************************************************************************/
/* Search a fixed length character array in memory for an occurance of    */
/* a match with a fixed length character array, and return a pointer.     */
/**************************************************************************/

char *MemSearch( char *pMem, int MemLen, char *pMatch, int MatchLen )
{
char *pMemPrev, *pMemFound;
int RemainLen;

  if( MemLen < 1 || MatchLen < 1 )
    return NULL;
  if( (RemainLen = MemLen - MatchLen + 1) < 1 )
    return NULL;

  pMemPrev = pMem;
  while( (pMemFound = memchr( pMemPrev, *pMatch, RemainLen )) != NULL )
    {
      if( MatchLen == 1 )
        return pMemFound;
      if( memcmp( pMemFound, pMatch, MatchLen ) == 0 )
        return pMemFound;
      RemainLen -= pMemFound - pMemPrev + 1;
      pMemPrev = pMemFound + 1;
    }
  return NULL;
}

