/*
 * IBM SupportPac MA0K
 * WebSphere MQ Completion Code and Reason Code analysis programs V1.4
 * http://www.ibm.com/support/docview.wss?uid=swg24000652
 *
 * MQRCP - WebSphere MQ Reason Code Print program
 *
 * "mqrcp.c" main program module in C
 *
 * Source file: \\IBM-L3V1408\WMQ-PROJ\MA0K-v1.4-Build\_Source\mqrcp-6.c
 *
 * Environments: Windows 32 bit, Unix (AIX, HP-UX, Solaris, Linux)
 *
 * This program accepts a numeric MQI Reason Code value on
 * the command line and displays the corresponding symbolic name.
 * It demonstrates the use of the MQ Code to Message conversion
 * functions in the "mqcm.c" module.
 * Its function is almost identical to the "mqrc" program supplied
 * with MQSeries version 5.2 and WebSphere MQ version 5.3 and later.
 *
 * Revision History
 * ================
 * Version  Date         Author    Changes
 *     1.1  24 Nov 2000  Glenn     Initial version
 *                       Baddeley <glenbadd@au1.ibm.com>
 *       2  15 Jan 2001  GB        Revision for version 1
 *                                 release 1 of Support Pac MA0K,
 *                                 Display MQCM_RC6MIN and MAX
 *       3  17 Jul 2002  GB        Mention WebSphere MQ
 *       4  28 Jan 2004  GB        Change copyright owner from IBM GSA to IBM Australia
 *       5  25 Oct 2005  GB        Update for WMQ 6.0,
 *                                 Support Windows 32 bit command prompt,
 *                                 Check command argument format, Accept lower case X
 *       6  25 Sep 2008  GB        Update for WMQ 7.0 GA
 *                                 Revise copyright and owner information
 */
#define SRCID "@(#)MQRCP.C VER=1.6 DATE=25-Sep-2008"

/*  <N_OCO_COPYRIGHT>
 *  Licensed Materials - Property of IBM
 *  (c) Copyright IBM Corp. 2008 All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *  <NOC_COPYRIGHT>                                                   */
#define OWNID "@(#)Licensed Materials - Property of IBM"
#define CPRID "@(#)Copyright 2008 Corp. All Rights Reserved"

#ifdef _WIN32
  #include <windows.h>
#else  /* Unix */
  #include <stdlib.h>
  #include <stdio.h>
  #include <string.h>
  #include <ctype.h>   /* For isdigit() */
#endif

/* Define MQLONG */
#include <cmqc.h>

/* MQ Code to Message conversion function prototypes */
#include "mqcmh.h"

static char SrcId[] = SRCID, CprId[] = CPRID, OwnId[] = OWNID;

/****************/
/* Main Program */
/****************/

int main( int ArgC, char *ArgP[] )
{
char *pCode, StrBuff[64];
MQLONG CodeNum;

  /* Check argument count on command line */
  if( ArgC < 2 )
    {
      printf( "WebSphere MQ Reason Code Print program\n%s\n%s, %s\n", SrcId+4, CprId+4, OwnId+4 );
      printf( "\nCommand line format is:\nmqrcp Code\n\n" );
      printf( "Code may be any decimal number or a hexadecimal number (prefix with X)\n" );
#ifdef MQRC_APPL_FIRST
      printf( "which is a MQI Reason Code (range %d to %d, %d to %d,\n",
              MQRC_APPL_FIRST, MQRC_APPL_LAST, MQCM_RCMIN, MQCM_RCMAX );
      printf( "%d to %d decimal).\n", MQCM_RC6MIN, MQCM_RC6MAX );
#else
      printf( "which is a MQI Reason Code (range %d to %d decimal).\n",
              MQCM_RCMIN, MQCM_RCMAX );
#endif
      exit( 1 );
    };

  /* Convert Reason Code on command line to an integer */
  pCode = ArgP[1];
  if( toupper( *pCode ) == 'X' )
    sscanf( pCode+1, "%x", &CodeNum );   /* hex */
  else if( isdigit( *pCode ) )
    sscanf( pCode, "%d", &CodeNum );   /* decimal */
  else
    {
      printf( "Argument '%s' must be decimal number or 'X' followed by a hexadecimal number\n",
              pCode );
      exit( 1 );
    }

  /* Display the symbolic name for the Reason Code */
  printf( "Reason Code %d x%X\n", CodeNum, CodeNum );
  printf( "%s\n", MQReasonCodeMsg( CodeNum, StrBuff ) );

  exit( 0 );
}

