Attribute VB_Name = "REGC"
'
' REGC - Windows Registry handling routines in Visual Basic 5.0
'
' "REGC"  Global Declarations and Code module
'
' Source file: \\IBM-L3V1408\MWARE-WMQ\winutils\vb50lib\regc9.bas
'
' Version   Date         Author     Changes
'     5.1   10 Oct 1999  Glenn      Initial Version
'                        Baddeley <glenbadd@au1.ibm.com>
'       2   20 Aug 2001  GB         Add description of functions
'       3   24 Aug 2001  GB         Rename functions, Add RegcOpenWriteCreate,
'                                   Add short names for hive keys, Add RegcFmtMsg,
'                                   Add RegcWriteValueLong and RegcWriteValueBin,
'                                   Add RegcPutWindowSize and RegcGetWindowSize
'       4    1 Oct 2002  GB         Add RegcTrace stuff, Add RegcEnumKeys
'       5    8 Sep 2003  GB         Add notices as "@(#)" constants
'       6    3 Nov 2004  GB         Add RegcDeleteKey
'       7   30 Jan 2005  GB         Error in RegcQueryValueLong returns default rather than 0,
'                                   RegcQueryValueString truncate at first null rather than last
'       8   12 Apr 2006  GB         Make RegcTrace public, Copy key for trace in RegcClose,
'                                   Add trace to Query, Write and Delete functions
'       9   14 Apr 2006  GB         Add TRACE local constants, Add RegcTraceInit
'
Private Const SRCID = "@(#)REGC9.BAS VER=5.9 DATE=14-Apr-2006"

' <N_OCO_COPYRIGHT>
' Licensed Materials - Property of IBM
' (c) Copyright IBM Corp. 2008 All Rights Reserved.
' US Government Users Restricted Rights - Use, duplication or
' disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
' <NOC_COPYRIGHT>
Private Const CPRID = "@(#)Copyright 2008 IBM Australia"
Private Const OWNID = "@(#)Licensed Materials - Property of IBM"

'
'========== Description of Functions ==========
'
' Sub RegcClose(RChKey As Long)
'   Close a key given the handle to the opened key.
'
' Function RegcOpenRead(ByVal RKHkey As Long, ByVal RKSubKey As String) As Long
'   Open a key for reading and return the handle, zero if error.
'
' Function RegcOpenWrite(ByVal WKHkey As Long, ByVal WKSubKey As String) As Long
'   Open a key for writing and return the handle, zero if error.
'
' Function RegcOpenWriteCreate(ByVal WKHkey As Long, ByVal WKSubKey As String) As Long
'   Open a key for writing, creating it if it doesn't exist, and return the handle, zero if error.
'
' Function RegcQueryValueString(ByVal RQhKey As Long, ByVal RQValue As String, ByVal RQDefaultData As String) As String
'   Returns the string data of the value.
'
' Function RegcQueryValueLong(ByVal RQLhKey As Long, ByVal RQLValue As String, ByVal RQLDefaultData As Long) As Long
'   Returns the long integer data of the value.
'
' Function RegcQueryValueBin(ByVal RQBhKey As Long, ByVal RQBValue As String) As String
'   Returns the binary data of the value.
'
' Function RegcEnumValues(ByVal REhKey As Long, REValue() As String, REData() As String, REType() As Long, REDataLen() As Long) As Integer
'   Enumerate the values of a key, and also the data, data type and data length.
'
' Function RegcEnumKeys(ByVal REhKey As Long, REValue() As String, REData() As String, REType() As Long, REDataLen() As Long) As Integer
'   Enumerate the sub-keys of a key, and also the data, data type and data length.
'
' Function RegcWriteValueString(ByVal RWhKey As Long, ByVal RWValue As String, ByVal RWData As String) As Long
'   Write string data for a value.  Returns 0 if success, RegcRetCode if failed.
'
' Function RegcWriteValueLong(ByVal WLhKey As Long, ByVal WLValue As string, ByVal WLData As long) As Long
'   Write long data for a value.  Returns 0 if success, RegcRetCode if failed.
'
' Function RegcWriteValueBin(ByVal WBhKey As Long, ByVal WBValue As String, ByVal WBData As String) As Long
'   Write binary data for a value.  Returns 0 if success, RegcRetCode if failed.
'
' Function RegcDeleteValue(ByVal RDhKey As Long, ByVal RDValue As String) As Long
'   Delete a value.  Returns 0 if success, RegcRetCode if failed.
'
' Function RegcDeleteKey(ByVal RDhKey As Long, ByVal RDSubKey As String) As Long
'   Delete a sub key.  Returns 0 if success, RegcRetCode if failed.
'
' Function RegcFmtMsg(ByVal RFMErrCode As Long)
'   Return the Windows error message for a Windows error code
'
' Sub RegcGetWindowSize(ByVal GWShKey As Long, ByVal GWSValue As String, GWSArgForm As Form)
'   Set a window size and position from a string in the registry
'
' Sub RegcPutWindowSize(ByVal PWShKey As Long, ByVal PWSValue As String, PWSArgForm As Form)
'   Store a window size and position into a string in the registry
'
Option Explicit

'========== Public data ==========

' Data content of a value
Public Const REGCMAXDATASIZE = 1024
Public RegcDataBuffer As String * REGCMAXDATASIZE
Public RegcDataLen As Long

' Name of a value
Public Const REGCMAXVALUESIZE = 1024
Public RegcValueBuffer As String * REGCMAXVALUESIZE
Public RegcValueLen As Long

' Type of a value. See "REG_xxx" Windows symbolic names.
Public RegcValueType As Long

' Returned code from Windows API registry functions.
' Codes are standard "ERROR_xxx" Windows symbolic names in winerror.h
Public RegcRetCode As Long

' Trace of recent calls to module
Public RegcTrace As String


'========== Public constants ==========

' Registry hive keys
Public Const HKEY_CLASSES_ROOT = &H80000000
Public Const HKEY_CURRENT_USER = &H80000001
Public Const HKEY_LOCAL_MACHINE = &H80000002
Public Const HKEY_USERS = &H80000003
Public Const HKEY_PERFORMANCE_DATA = &H80000004
Public Const HKEY_CURRENT_CONFIG = &H80000005
Public Const HKEY_DYN_DATA = &H80000006

' Registry hive keys with short names
Public Const HKCR = &H80000000
Public Const HKCU = &H80000001
Public Const HKLM = &H80000002
Public Const HKUS = &H80000003
Public Const HKPDATA = &H80000004
Public Const HKCC = &H80000005
Public Const HKDD = &H80000006

' Registry hive key literals
Public Const REGSEP = "\"   ' Key name separator
Public Const HKCR_Lit = "HKEY_CLASSES_ROOT"
Public Const HKCU_Lit = "HKEY_CURRENT_USER"
Public Const HKLM_Lit = "HKEY_LOCAL_MACHINE"
Public Const HKUS_Lit = "HKEY_USERS"
Public Const HKCC_Lit = "HKEY_CURENT_CONFIG"
Public Const HKDD_Lit = "HKEY_DYN_DATA"

' Registry Data Types (returned in RegcValueType)
Public Const REG_NONE = 0                       ' No value type
Public Const REG_SZ = 1                         ' Unicode nul terminated string
Public Const REG_EXPAND_SZ = 2                  ' Unicode nul terminated string
Public Const REG_BINARY = 3                     ' Free form binary
Public Const REG_DWORD = 4                      ' 32-bit number
Public Const REG_DWORD_LITTLE_ENDIAN = 4        ' 32-bit number (same as REG_DWORD)
Public Const REG_DWORD_BIG_ENDIAN = 5           ' 32-bit number
Public Const REG_LINK = 6                       ' Symbolic Link (unicode)
Public Const REG_MULTI_SZ = 7                   ' Multiple Unicode strings

'========== Local data ==========

Private hOpenedKey As Long
Private ErrMsg As String

'========== Local constants =========

Private Const TRACEMAXLEN = 32000
Private Const TRACETRIMLEN = 256
Private Const TRACEBACKLEN = 512

' General Windows API constants from "winerror.h"

Private Const ERROR_SUCCESS = 0&
Private Const ERROR_FILE_NOT_FOUND = 2&
Private Const ERROR_MORE_DATA = 234&
Private Const ERROR_NO_MORE_ITEMS = 259&

' Constants for Registry functions

Private Const SYNCHRONIZE = &H100000
Private Const READ_CONTROL = &H20000
Private Const STANDARD_RIGHTS_ALL = &H1F0000
Private Const STANDARD_RIGHTS_READ = (READ_CONTROL)
Private Const STANDARD_RIGHTS_WRITE = (READ_CONTROL)

Private Const KEY_QUERY_VALUE = &H1
Private Const KEY_SET_VALUE = &H2
Private Const KEY_CREATE_SUB_KEY = &H4
Private Const KEY_ENUMERATE_SUB_KEYS = &H8
Private Const KEY_EVENT = &H1
Private Const KEY_NOTIFY = &H10
Private Const KEY_CREATE_LINK = &H20
Private Const KEY_ALL_ACCESS = ((STANDARD_RIGHTS_ALL Or KEY_QUERY_VALUE Or KEY_SET_VALUE Or KEY_CREATE_SUB_KEY Or KEY_ENUMERATE_SUB_KEYS Or KEY_NOTIFY Or KEY_CREATE_LINK) And (Not SYNCHRONIZE))
Private Const KEY_READ = ((STANDARD_RIGHTS_READ Or KEY_QUERY_VALUE Or KEY_ENUMERATE_SUB_KEYS Or KEY_NOTIFY) And (Not SYNCHRONIZE))
Private Const KEY_EXECUTE = ((KEY_READ) And (Not SYNCHRONIZE))
Private Const KEY_WRITE = ((STANDARD_RIGHTS_WRITE Or KEY_SET_VALUE Or KEY_CREATE_SUB_KEY) And (Not SYNCHRONIZE))

' Key creation options from "winnt.h"
Private Const REG_OPTION_RESERVED = 0            ' Parameter is reserved
Private Const REG_OPTION_NON_VOLATILE = 0        ' Key is preserved when system is rebooted
Private Const REG_OPTION_VOLATILE = 1            ' Key is not preserved when system is rebooted
Private Const REG_OPTION_CREATE_LINK = 2         ' Created key is a symbolic link
Private Const REG_OPTION_BACKUP_RESTORE = 4      ' open for backup or restore, special access rules, privilege required
Private Const REG_OPTION_OPEN_LINK = 8           ' symbolic link

' Constants for FormatMessage
Private Const FORMAT_MESSAGE_ALLOCATE_BUFFER = &H100
Private Const FORMAT_MESSAGE_FROM_SYSTEM = &H1000
Private Const LANG_NEUTRAL = &H0
Private Const SUBLANG_DEFAULT = &H1

' Windows Advanced API declarations
' Unused declarations are commented out

' Registry functions
Private Declare Function RegCloseKey Lib "advapi32.dll" (ByVal hKey As Long) As Long
'Private Declare Function RegCreateKey Lib "advapi32.dll" Alias "RegCreateKeyA" (ByVal hKey As Long, ByVal lpSubKey As String, phkResult As Long) As Long
Private Declare Function RegCreateKeyEx Lib "advapi32.dll" Alias "RegCreateKeyExA" (ByVal hKey As Long, ByVal lpSubKey As String, ByVal Reserved As Long, ByVal lpClass As String, ByVal dwOptions As Long, ByVal samDesired As Long, lpSecurityAttributes As Any, phkResult As Long, lpdwDisposition As Long) As Long
Private Declare Function RegDeleteKey Lib "advapi32.dll" Alias "RegDeleteKeyA" (ByVal hKey As Long, ByVal lpSubKey As String) As Long
Private Declare Function RegDeleteValue Lib "advapi32.dll" Alias "RegDeleteValueA" (ByVal hKey As Long, ByVal lpValueName As String) As Long
Private Declare Function RegEnumKey Lib "advapi32.dll" Alias "RegEnumKeyA" (ByVal hKey As Long, ByVal dwIndex As Long, ByVal lpName As String, ByVal cbName As Long) As Long
'Private Declare Function RegEnumKeyEx Lib "advapi32.dll" Alias "RegEnumKeyExA" (ByVal hKey As Long, ByVal dwIndex As Long, ByVal lpName As String, lpcbName As Long, lpReserved As Long, ByVal lpClass As String, lpcbClass As Long, lpftLastWriteTime As FILETIME) As Long
Private Declare Function RegEnumValue Lib "advapi32.dll" Alias "RegEnumValueA" (ByVal hKey As Long, ByVal dwIndex As Long, ByVal lpValueName As String, lpcbValueName As Long, ByVal lpReserved As Long, lpType As Long, ByVal lpData As String, lpcbData As Long) As Long
Private Declare Function RegFlushKey Lib "advapi32.dll" (ByVal hKey As Long) As Long
'Private Declare Function RegLoadKey Lib "advapi32.dll" Alias "RegLoadKeyA" (ByVal hKey As Long, ByVal lpSubKey As String, ByVal lpFile As String) As Long
'Private Declare Function RegNotifyChangeKeyValue Lib "advapi32.dll" (ByVal hKey As Long, ByVal bWatchSubtree As Long, ByVal dwNotifyFilter As Long, ByVal hEvent As Long, ByVal fAsynchronus As Long) As Long
'Private Declare Function RegOpenKey Lib "advapi32.dll" Alias "RegOpenKeyA" (ByVal hKey As Long, ByVal lpSubKey As String, phkResult As Long) As Long
Private Declare Function RegOpenKeyEx Lib "advapi32.dll" Alias "RegOpenKeyExA" (ByVal hKey As Long, ByVal lpSubKey As String, ByVal ulOptions As Long, ByVal samDesired As Long, phkResult As Long) As Long
'Private Declare Function RegQueryInfoKey Lib "advapi32.dll" Alias "RegQueryInfoKeyA" (ByVal hKey As Long, ByVal lpClass As String, lpcbClass As Long, lpReserved As Long, lpcSubKeys As Long, lpcbMaxSubKeyLen As Long, lpcbMaxClassLen As Long, lpcValues As Long, lpcbMaxValueNameLen As Long, lpcbMaxValueLen As Long, lpcbSecurityDescriptor As Long, lpftLastWriteTime As FILETIME) As Long
'Private Declare Function RegQueryValue Lib "advapi32.dll" Alias "RegQueryValueA" (ByVal hKey As Long, ByVal lpSubKey As String, ByVal lpValue As String, lpcbValue As Long) As Long
Private Declare Function RegQueryValueEx Lib "advapi32.dll" Alias "RegQueryValueExA" (ByVal hKey As Long, ByVal lpValueName As String, ByVal lpReserved As Long, lpType As Long, ByVal lpData As String, lpcbData As Long) As Long
Private Declare Function RegQueryValueExLong Lib "advapi32.dll" Alias "RegQueryValueExA" (ByVal hKey As Long, ByVal lpValueName As String, ByVal lpReserved As Long, lpType As Long, lpData As Long, lpcbData As Long) As Long
'Private Declare Function RegReplaceKey Lib "advapi32.dll" Alias "RegReplaceKeyA" (ByVal hKey As Long, ByVal lpSubKey As String, ByVal lpNewFile As String, ByVal lpOldFile As String) As Long
'Private Declare Function RegRestoreKey Lib "advapi32.dll" Alias "RegRestoreKeyA" (ByVal hKey As Long, ByVal lpFile As String, ByVal dwFlags As Long) As Long
'Private Declare Function RegSaveKey Lib "advapi32.dll" Alias "RegSaveKeyA" (ByVal hKey As Long, ByVal lpFile As String, lpSecurityAttributes As SECURITY_ATTRIBUTES) As Long
'Private Declare Function RegSetValue Lib "advapi32.dll" Alias "RegSetValueA" (ByVal hKey As Long, ByVal lpSubKey As String, ByVal dwType As Long, ByVal lpData As String, ByVal cbData As Long) As Long
'Private Declare Function RegSetKeySecurity Lib "advapi32.dll" (ByVal hKey As Long, ByVal SecurityInformation As Long, pSecurityDescriptor As SECURITY_DESCRIPTOR) As Long
Private Declare Function RegSetValueExS Lib "advapi32.dll" Alias "RegSetValueExA" (ByVal hKey As Long, ByVal lpValueName As String, ByVal Reserved As Long, ByVal dwType As Long, ByVal lpData As String, ByVal cbData As Long) As Long
Private Declare Function RegSetValueExL Lib "advapi32.dll" Alias "RegSetValueExA" (ByVal hKey As Long, ByVal lpValueName As String, ByVal Reserved As Long, ByVal dwType As Long, lpData As Long, ByVal cbData As Long) As Long
'Private Declare Function RegUnLoadKey Lib "advapi32.dll" Alias "RegUnLoadKeyA" (ByVal hKey As Long, ByVal lpSubKey As String) As Long

' Windows Kernel API declarations
Private Declare Function FormatMessage Lib "kernel32" Alias "FormatMessageA" (ByVal dwFlags As Long, lpSource As Any, ByVal dwMessageId As Long, ByVal dwLanguageId As Long, ByVal lpBuffer As String, ByVal nSize As Long, Arguments As Long) As Long

Public Sub RegcClose(RChKey As Long)

Dim RChKeyCopy As Long

    RChKeyCopy = RChKey
    If RChKey <> 0 Then
        RegcRetCode = RegFlushKey(RChKey)
        If RegcRetCode <> ERROR_SUCCESS Then
            ErrMsg = "Unexpected error from Windows API." & vbCrLf
            ErrMsg = ErrMsg & "RegFlushKey( ) returned " & RegcFmtMsg(RegcRetCode) & vbCrLf
            ErrMsg = ErrMsg & "RChKey=" & RChKey & vbCrLf
            ErrMsg = ErrMsg & RegcTraceBack
            MsgBox ErrMsg, vbExclamation, "REGC RegcClose( )"
        End If
        RegcRetCode = RegCloseKey(RChKey)
        If RegcRetCode <> ERROR_SUCCESS Then
            ErrMsg = "Unexpected error from Windows API." & vbCrLf
            ErrMsg = ErrMsg & "RegCloseKey( ) returned " & RegcFmtMsg(RegcRetCode) & vbCrLf
            ErrMsg = ErrMsg & "RChKey=" & RChKey & vbCrLf
            ErrMsg = ErrMsg & RegcTraceBack
            MsgBox ErrMsg, vbExclamation, "REGC RegcClose( )"
        Else
            RChKey = 0      ' Success, make the key invalid
        End If
    End If
    RegcTraceAdd "Close(" & RChKeyCopy & ")"
    
End Sub

Public Function RegcOpenRead(ByVal RKHkey As Long, ByVal RKSubKey As String) As Long

    If Len(RegcTrace) = 0 Then RegcTraceInit
    RegcRetCode = RegOpenKeyEx(RKHkey, RKSubKey, 0&, KEY_READ, hOpenedKey)
    Select Case RegcRetCode
      Case ERROR_SUCCESS
        ' Key is in hOpenedKey
      Case ERROR_FILE_NOT_FOUND
        hOpenedKey = 0
      Case Else
        ErrMsg = "Unexpected error from Windows API." & vbCrLf
        ErrMsg = ErrMsg & "RegOpenKeyEx( ) returned " & RegcFmtMsg(RegcRetCode) & vbCrLf
        ErrMsg = ErrMsg & "RKHKey=" & RKHkey & " RKSubKey='" & RKSubKey & "'" & vbCrLf
        ErrMsg = ErrMsg & RegcTraceBack
        MsgBox ErrMsg, vbExclamation, "REGC RegcOpenRead( )"
        hOpenedKey = 0
    End Select
    RegcTraceAdd "OpenRead(" & RKHkey & ",'" & RKSubKey & "') hOpenedKey=" & hOpenedKey
    RegcOpenRead = hOpenedKey
    
End Function

Public Function RegcOpenWrite(ByVal WKHkey As Long, ByVal WKSubKey As String) As Long

    If Len(RegcTrace) = 0 Then RegcTraceInit
    RegcRetCode = RegOpenKeyEx(WKHkey, WKSubKey, 0&, KEY_ALL_ACCESS, hOpenedKey)
    Select Case RegcRetCode
      Case ERROR_SUCCESS
        ' Key is in hOpenedKey
      Case ERROR_FILE_NOT_FOUND
        hOpenedKey = 0
      Case Else
        ErrMsg = "Unexpected error from Windows API." & vbCrLf
        ErrMsg = ErrMsg & "RegOpenKeyEx( ) returned " & RegcFmtMsg(RegcRetCode) & vbCrLf
        ErrMsg = ErrMsg & "WKHKey=" & WKHkey & " WKSubKey='" & WKSubKey & "'" & vbCrLf
        ErrMsg = ErrMsg & RegcTraceBack
        MsgBox ErrMsg, vbExclamation, "REGC RegcOpenWrite( )"
        hOpenedKey = 0
    End Select
    RegcTraceAdd "OpenWrite(" & WKHkey & ",'" & WKSubKey & "') hOpenedKey=" & hOpenedKey
    RegcOpenWrite = hOpenedKey
    
End Function

Public Function RegcOpenWriteCreate(ByVal WKHkey As Long, ByVal WKSubKey As String) As Long

Dim CrDisposition As Long

    If Len(RegcTrace) = 0 Then RegcTraceInit
    RegcRetCode = RegOpenKeyEx(WKHkey, WKSubKey, 0&, KEY_ALL_ACCESS, hOpenedKey)
    Select Case RegcRetCode
      Case ERROR_SUCCESS
        ' Key is in hOpenedKey
      Case ERROR_FILE_NOT_FOUND
        GoSub OpenWriteCreate
      Case Else
        ErrMsg = "Unexpected error from Windows API." & vbCrLf
        ErrMsg = ErrMsg & "RegOpenKeyEx( ) returned " & RegcFmtMsg(RegcRetCode) & vbCrLf
        ErrMsg = ErrMsg & "WKHKey=" & WKHkey & " WKSubKey='" & WKSubKey & "'" & vbCrLf
        ErrMsg = ErrMsg & RegcTraceBack
        MsgBox ErrMsg, vbExclamation, "REGC RegcOpenWriteCreate( )"
        hOpenedKey = 0
    End Select
    RegcTraceAdd "OpenWriteCreate(" & WKHkey & ",'" & WKSubKey & "') hOpenedKey=" & hOpenedKey
    RegcOpenWriteCreate = hOpenedKey
    
Exit Function

OpenWriteCreate:
        
    RegcRetCode = RegCreateKeyEx(WKHkey, WKSubKey, 0&, "REG_DWORD", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, ByVal 0&, hOpenedKey, CrDisposition)
    Select Case RegcRetCode
      Case ERROR_SUCCESS
        ' Key is in hOpenedKey
      Case Else
        ErrMsg = "Unexpected error from Windows API." & vbCrLf
        ErrMsg = ErrMsg & "RegCreateKeyEx( ) returned " & RegcFmtMsg(RegcRetCode) & vbCrLf
        ErrMsg = ErrMsg & "WKHKey=" & WKHkey & " WKSubKey='" & WKSubKey & "'"
        MsgBox ErrMsg, vbExclamation, "REGC RegcOpenWriteCreate( )"
        hOpenedKey = 0
    End Select

Return
    
End Function

Public Function RegcQueryValueString(ByVal RQhKey As Long, ByVal RQValue As String, ByVal RQDefaultData As String) As String

Dim RQDataString As String

    RegcDataLen = REGCMAXDATASIZE
    RegcRetCode = RegQueryValueEx(RQhKey, RQValue, 0&, RegcValueType, RegcDataBuffer, RegcDataLen)
    Select Case RegcRetCode
      Case ERROR_SUCCESS
        Mid(RegcDataBuffer, RegcDataLen + 1, 1) = vbNullChar  ' Guarantee a null terminator
        RQDataString = Left(RegcDataBuffer, InStr(RegcDataBuffer, vbNullChar) - 1)
      Case ERROR_FILE_NOT_FOUND
        RQDataString = RQDefaultData      ' Return the default data
      Case ERROR_MORE_DATA
        RegcDataLen = -RegcDataLen          ' -ve indicates value too long
        RQDataString = "?DataTooLong"
      Case Else
        ErrMsg = "Unexpected error from Windows API." & vbCrLf
        ErrMsg = ErrMsg & "RegQueryValueEx( ) returned " & RegcFmtMsg(RegcRetCode) & vbCrLf
        ErrMsg = ErrMsg & "RQhKey=" & RQhKey & vbCrLf & "RQValue='" & RQValue & "'" & vbCrLf
        ErrMsg = ErrMsg & "RegcValueType=" & RegcValueType & vbCrLf & "RegcDataLen=" & RegcDataLen & vbCrLf
        ErrMsg = ErrMsg & RegcTraceBack
        MsgBox ErrMsg, vbExclamation, "REGC RegcQueryValue( )"
        RQDataString = "?Error " & RegcRetCode
    End Select
    RegcTraceAdd "QueryValueString(" & RQhKey & ",'" & RQValue & "','" & RQDefaultData & "') returns '" & RQDataString & "' RegcDataLen=" & RegcDataLen
    RegcQueryValueString = RQDataString

End Function

Public Function RegcQueryValueLong(ByVal RQLhKey As Long, ByVal RQLValue As String, ByVal RQLDefaultData As Long) As Long

Dim RQLDataLong As Long

    RegcDataLen = 4
    RegcRetCode = RegQueryValueExLong(RQLhKey, RQLValue, 0&, RegcValueType, RQLDataLong, RegcDataLen)
    Select Case RegcRetCode
      Case ERROR_SUCCESS
        ' Data is in RQLDataLong
      Case ERROR_FILE_NOT_FOUND
        RQLDataLong = RQLDefaultData      ' Return the default data
      Case Else
        ErrMsg = "Unexpected error from Windows API." & vbCrLf
        ErrMsg = ErrMsg & "RegQueryValueEx( ) returned " & RegcFmtMsg(RegcRetCode) & vbCrLf
        ErrMsg = ErrMsg & "RQLhKey=" & RQLhKey & vbCrLf & "RQLValue='" & RQLValue & "'" & vbCrLf
        ErrMsg = ErrMsg & "RegcValueType=" & RegcValueType & vbCrLf & "RegcDataLen=" & RegcDataLen & vbCrLf
        ErrMsg = ErrMsg & RegcTraceBack
        MsgBox ErrMsg, vbExclamation, "REGC RegcQueryValueLong( )"
        RQLDataLong = RQLDefaultData      ' Return the default data
    End Select
    RegcTraceAdd "QueryValueLong(" & RQLhKey & "," & RQLValue & "," & RQLDefaultData & ") returns " & RQLDataLong
    RegcQueryValueLong = RQLDataLong

End Function

Public Function RegcQueryValueBin(ByVal RQBhKey As Long, ByVal RQBValue As String) As String

    RegcDataLen = REGCMAXDATASIZE
    RegcRetCode = RegQueryValueEx(RQBhKey, RQBValue, 0&, RegcValueType, RegcDataBuffer, RegcDataLen)
    Select Case RegcRetCode
      Case ERROR_SUCCESS
        RegcQueryValueBin = Left(RegcDataBuffer, RegcDataLen)
      Case ERROR_FILE_NOT_FOUND
        RegcQueryValueBin = ""    ' Return nothing
        RegcDataLen = -1
      Case Else
        ErrMsg = "Unexpected error from Windows API." & vbCrLf
        ErrMsg = ErrMsg & "RegQueryValueEx( ) returned " & RegcFmtMsg(RegcRetCode) & vbCrLf
        ErrMsg = ErrMsg & "RQBhKey=" & RQBhKey & vbCrLf & "RQBValue='" & RQBValue & "'" & vbCrLf
        ErrMsg = ErrMsg & "RegcValueType=" & RegcValueType & vbCrLf & "RegcDataLen=" & RegcDataLen & vbCrLf
        ErrMsg = ErrMsg & RegcTraceBack
        MsgBox ErrMsg, vbExclamation, "REGC RegcQueryValueBin( )"
        RegcQueryValueBin = ""    ' Return nothing
        RegcDataLen = -1
    End Select
    RegcTraceAdd "QueryValueBin(" & RQBhKey & ",'" & RQBValue & "') RegcDataLen=" & RegcDataLen

End Function

Public Function RegcEnumValues(ByVal REhKey As Long, REValue() As String, REData() As String, REType() As Long, REDataLen() As Long) As Integer

Dim REIdx As Long

  REIdx = 0
  Do
    RegcValueLen = REGCMAXVALUESIZE
    RegcDataLen = REGCMAXDATASIZE
    RegcRetCode = RegEnumValue(REhKey, REIdx, RegcValueBuffer, RegcValueLen, 0&, RegcValueType, RegcDataBuffer, RegcDataLen)
    Select Case RegcRetCode
      Case ERROR_SUCCESS
        REIdx = REIdx + 1
        REValue(REIdx) = Left(RegcValueBuffer, RegcValueLen)
        Select Case RegcValueType
          Case REG_SZ, REG_EXPAND_SZ
            If RegcValueLen > 0 Then
                Do   ' Dont include trailing nulls on string data
                    If RegcDataLen = 0 Then
                        REData(REIdx) = ""
                        Exit Do
                    End If
                    If Mid(RegcDataBuffer, RegcDataLen, 1) <> vbNullChar Then
                        REData(REIdx) = Left(RegcDataBuffer, RegcDataLen)
                        Exit Do
                    End If
                    RegcDataLen = RegcDataLen - 1
                Loop
            Else
                REData(REIdx) = ""
            End If
          Case Else
            REData(REIdx) = Left(RegcDataBuffer, RegcDataLen)
        End Select
        REType(REIdx) = RegcValueType
        REDataLen(REIdx) = RegcDataLen
      Case ERROR_NO_MORE_ITEMS
        Exit Do
      Case Else
        ErrMsg = "Unexpected error from Windows API." & vbCrLf
        ErrMsg = ErrMsg & "RegEnumValue( ) returned " & RegcFmtMsg(RegcRetCode) & vbCrLf
        ErrMsg = ErrMsg & "REhKey=" & REhKey & vbCrLf & "REIdx=" & REIdx & vbCrLf
        ErrMsg = ErrMsg & "RegcValueType=" & RegcValueType & vbCrLf & "RegcValueLen=" & RegcValueLen & vbCrLf & "RegcDataLen=" & RegcDataLen
        ErrMsg = ErrMsg & RegcTraceBack
        MsgBox ErrMsg, vbExclamation, "REGC RegcEnumValues( )"
        REIdx = -1
        Exit Do
    End Select
  Loop
  RegcTraceAdd "EnumValues(" & REhKey & ",Vals(),Data(),Type(),DataLen()) returns " & REIdx & " items"
  RegcEnumValues = REIdx
    
End Function

Public Function RegcEnumKeys(ByVal REhKey As Long, REKey() As String) As Integer

Dim REIdx As Long, NullPos As Integer

  REIdx = 0
  Do
    RegcValueLen = REGCMAXVALUESIZE
    RegcRetCode = RegEnumKey(REhKey, REIdx, RegcValueBuffer, RegcValueLen)
    Select Case RegcRetCode
      Case ERROR_SUCCESS
        REIdx = REIdx + 1
        NullPos = InStr(RegcValueBuffer, vbNullChar)
        If NullPos Then
            REKey(REIdx) = Left(RegcValueBuffer, NullPos - 1)
        Else
            REKey(REIdx) = RegcValueBuffer
        End If
      Case ERROR_NO_MORE_ITEMS
        Exit Do
      Case Else
        ErrMsg = "Unexpected error from Windows API." & vbCrLf
        ErrMsg = ErrMsg & "RegEnumKey( ) returned " & RegcFmtMsg(RegcRetCode) & vbCrLf
        ErrMsg = ErrMsg & "REhKey=" & REhKey & vbCrLf & "REIdx=" & REIdx & vbCrLf
        ErrMsg = ErrMsg & "RegcValueType=" & RegcValueType & vbCrLf & "RegcValueLen=" & RegcValueLen & vbCrLf & "RegcDataLen=" & RegcDataLen
        ErrMsg = ErrMsg & RegcTraceBack
        MsgBox ErrMsg, vbExclamation, "REGC RegcEnumValues( )"
        REIdx = -1
        Exit Do
    End Select
  Loop
  RegcTraceAdd "EnumKeys(" & REhKey & ",Key()) returns " & REIdx & " items"
  RegcEnumKeys = REIdx
    
End Function

Public Function RegcWriteValueString(ByVal WShKey As Long, ByVal WSValue As String, ByVal WSData As String) As Long

    RegcDataLen = Len(WSData)
    RegcValueType = REG_SZ
    RegcRetCode = RegSetValueExS(WShKey, WSValue, 0&, RegcValueType, WSData, RegcDataLen)
    Select Case RegcRetCode
      Case ERROR_SUCCESS
        RegcWriteValueString = 0
      Case Else
        ErrMsg = "Unexpected error from Windows API." & vbCrLf
        ErrMsg = ErrMsg & "RegSetValueEx( ) returned " & RegcFmtMsg(RegcRetCode) & vbCrLf
        ErrMsg = ErrMsg & "WShKey=" & WShKey & vbCrLf & "WSValue='" & WSValue & "'" & vbCrLf
        ErrMsg = ErrMsg & "WSData='" & WSData & "'" & vbCrLf
        ErrMsg = ErrMsg & "RegcValueType=" & RegcValueType & vbCrLf & "RegcDataLen=" & RegcDataLen & vbCrLf
        ErrMsg = ErrMsg & RegcTraceBack
        MsgBox ErrMsg, vbExclamation, "REGC RegcWriteValueString( )"
        RegcWriteValueString = RegcRetCode
    End Select
    RegcTraceAdd "WriteValueString(" & WShKey & ",'" & WSValue & "','" & WSData & "')"

End Function

Public Function RegcWriteValueLong(ByVal WLhKey As Long, ByVal WLValue As String, ByVal WLData As Long) As Long

    RegcDataLen = 4
    RegcValueType = REG_DWORD
    RegcRetCode = RegSetValueExL(WLhKey, WLValue, 0&, RegcValueType, WLData, RegcDataLen)
    Select Case RegcRetCode
      Case ERROR_SUCCESS
        RegcWriteValueLong = 0
      Case Else
        ErrMsg = "Unexpected error from Windows API." & vbCrLf
        ErrMsg = ErrMsg & "RegSetValueEx( ) returned " & RegcFmtMsg(RegcRetCode) & vbCrLf
        ErrMsg = ErrMsg & "WLhKey=" & WLhKey & vbCrLf & "WLValue='" & WLValue & "'" & vbCrLf
        ErrMsg = ErrMsg & "RegcValueType=" & RegcValueType & vbCrLf & "RegcDataLen=" & RegcDataLen & vbCrLf
        ErrMsg = ErrMsg & RegcTraceBack
        MsgBox ErrMsg, vbExclamation, "REGC RegcWriteValueLong( )"
        RegcWriteValueLong = RegcRetCode
    End Select
    RegcTraceAdd "WriteValueLong(" & WLhKey & ",'" & WLValue & "'," & WLValue & ")"

End Function

Public Function RegcWriteValueBin(ByVal WBhKey As Long, ByVal WBValue As String, ByVal WBData As String) As Long

    RegcDataLen = Len(WBData)
    RegcValueType = REG_BINARY
    RegcRetCode = RegSetValueExS(WBhKey, WBValue, 0&, RegcValueType, WBData, RegcDataLen)
    Select Case RegcRetCode
      Case ERROR_SUCCESS
        RegcWriteValueBin = 0
      Case Else
        ErrMsg = "Unexpected error from Windows API." & vbCrLf
        ErrMsg = ErrMsg & "RegSetValueEx( ) returned " & RegcFmtMsg(RegcRetCode) & vbCrLf
        ErrMsg = ErrMsg & "WBhKey=" & WBhKey & vbCrLf & "WBValue='" & WBValue & "'" & vbCrLf
        ErrMsg = ErrMsg & "RegcValueType=" & RegcValueType & vbCrLf & "RegcDataLen=" & RegcDataLen & vbCrLf
        ErrMsg = ErrMsg & RegcTraceBack
        MsgBox ErrMsg, vbExclamation, "REGC RegcWriteValueBin( )"
        RegcWriteValueBin = RegcRetCode
    End Select
    RegcTraceAdd "WriteValueBin(" & WBhKey & ",'" & WBValue & "',[BinaryData]) RegcDataLen=" & RegcDataLen

End Function

Public Function RegcDeleteValue(ByVal RDhKey As Long, ByVal RDValue As String) As Long

    RegcRetCode = RegDeleteValue(RDhKey, RDValue)
    Select Case RegcRetCode
      Case ERROR_SUCCESS, ERROR_FILE_NOT_FOUND      ' Ignore error if value does not exist
        RegcDeleteValue = 0
      Case Else
        ErrMsg = "Unexpected error from Windows API." & vbCrLf
        ErrMsg = ErrMsg & "RegDeleteValue( ) returned " & RegcFmtMsg(RegcRetCode) & vbCrLf
        ErrMsg = ErrMsg & "RDhKey=" & RDhKey & vbCrLf & "RDValue='" & RDValue & "'" & vbCrLf
        ErrMsg = ErrMsg & RegcTraceBack
        MsgBox ErrMsg, vbExclamation, "REGC RegcDeleteValue( )"
        RegcDeleteValue = RegcRetCode
    End Select
    RegcTraceAdd "DeleteValue(" & RDhKey & ",'" & RDValue & "')"

End Function

Public Function RegcDeleteKey(ByVal RDhKey As Long, ByVal RDSubKey As String) As Long

    RegcRetCode = RegDeleteKey(RDhKey, RDSubKey)
    Select Case RegcRetCode
      Case ERROR_SUCCESS
        RegcDeleteKey = 0
      Case Else
        ErrMsg = "Unexpected error from Windows API." & vbCrLf
        ErrMsg = ErrMsg & "RegDeleteKey( ) returned " & RegcFmtMsg(RegcRetCode) & vbCrLf
        ErrMsg = ErrMsg & "RDhKey=" & RDhKey & vbCrLf & "RDSubKey='" & RDSubKey & "'" & vbCrLf
        ErrMsg = ErrMsg & RegcTraceBack
        MsgBox ErrMsg, vbExclamation, "REGC RegcDeleteKey( )"
        RegcDeleteKey = RegcRetCode
    End Select
    RegcTraceAdd "DeleteKey(" & RDhKey & ",'" & RDSubKey & "')"

End Function

Public Function RegcFmtMsg(ByVal RFMErrCode As Long)

Const MSGBUFFERSIZE = 256
Dim MsgBuffer As String * MSGBUFFERSIZE
Dim ChPos As Integer, RetLen As Long
    
    RetLen = FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, ByVal 0&, RFMErrCode, LANG_NEUTRAL, MsgBuffer, MSGBUFFERSIZE, ByVal 0&)
    If RetLen < 1 Then
        ErrMsg = "Unexpected error from Windows API." & vbCrLf
        ErrMsg = ErrMsg & "FormatMessage( ) returned " & RetLen & vbCrLf
        ErrMsg = ErrMsg & "RFMErrCode=" & RFMErrCode & vbCrLf
        MsgBox ErrMsg, vbExclamation, "REGC RegcFmtMsg( )"
        RegcFmtMsg = RFMErrCode & ": FormatMessage() failed!"
    ElseIf Mid(MsgBuffer, RetLen + 1, 1) = vbNullChar Then
        If Mid(MsgBuffer, RetLen - 1, 2) = vbCrLf Then
            RegcFmtMsg = RFMErrCode & ": " & Left(MsgBuffer, RetLen - 2)
        Else
            RegcFmtMsg = RFMErrCode & ": " & Left(MsgBuffer, RetLen)
        End If
    Else
        RegcFmtMsg = RFMErrCode & ": " & MsgBuffer
    End If
    
End Function

Public Sub RegcGetWindowSize(ByVal GWShKey As Long, ByVal GWSValue As String, GWSArgForm As Form)

Const VALSEP = ","
Const DEFVAL = "0,0,0,0"
Dim KeyValues As String, PrevPos As Integer, ChPos As Integer, Idx As Integer

    KeyValues = RegcQueryValueString(GWShKey, GWSValue, DEFVAL) & VALSEP
    If RegcRetCode Then Exit Sub
    ReDim WinSize(1 To 4) As Single
    PrevPos = 1
    For Idx = 1 To 4
        ChPos = InStr(PrevPos, KeyValues, VALSEP)
        WinSize(Idx) = CSng(Mid(KeyValues, PrevPos, ChPos - PrevPos))
        PrevPos = ChPos + 1
    Next Idx
    If WinSize(4) <> 0 Then
        GWSArgForm.Move WinSize(1), WinSize(2)
        GWSArgForm.Width = WinSize(3)
        GWSArgForm.Height = WinSize(4)
    End If

End Sub

Public Sub RegcPutWindowSize(ByVal PWShKey As Long, ByVal PWSValue As String, PWSArgForm As Form)

Const VALSEP = ","
Dim PWSData As String

    If PWSArgForm.WindowState = vbNormal Then
        PWSData = PWSArgForm.Left & VALSEP & PWSArgForm.Top & VALSEP & PWSArgForm.Width & VALSEP & PWSArgForm.Height
        RegcWriteValueString PWShKey, PWSValue, PWSData
    End If
    
End Sub

Private Sub RegcTraceAdd(ByVal TrStr As String)

Dim ChPos As Integer

    RegcTrace = RegcTrace & TrStr & " RC=" & RegcRetCode & vbCrLf
    If Len(RegcTrace) > TRACEMAXLEN Then
        ChPos = InStr(TRACETRIMLEN, RegcTrace, vbCrLf)
        RegcTrace = Now & " Trace starts" & Right(RegcTrace, ChPos)
    End If
    
End Sub

Private Sub RegcTraceInit()

    RegcTrace = Now & " Trace starts" & vbCrLf
    
End Sub

Public Function RegcTraceBack() As String

Dim BegPos As Integer, EndPos As Integer, BackCnt As Integer
Dim Msg As String

    Msg = "Trace back:" & vbCrLf
    ' Concatenate the last 10 entries in reverse order
    EndPos = Len(RegcTrace) - 1
    BegPos = EndPos - 1
    BackCnt = 0
    Do
        If BegPos < 1 Then
            Msg = Msg & BegPos & " " & Left(RegcTrace, EndPos - 1) & vbCrLf
            Exit Do
        End If
        If Mid(RegcTrace, BegPos, 1) = vbCr Then
            ' cr lf a b c d cr lf
            ' ^begpos       ^endpos
            Msg = Msg & Mid(RegcTrace, BegPos + 2, EndPos - BegPos - 2) & vbCrLf
            EndPos = BegPos
            BackCnt = BackCnt + 1
            If BackCnt = 10 Then Exit Do
        End If
        BegPos = BegPos - 1
    Loop
    RegcTraceBack = Msg

End Function
